@@ -1,19 +1,32 @@
-import os
-import sys
-import platform
 import logging
+import os
+import platform
+import sys
 from collections import OrderedDict
-from typing import Optional, List
+from typing import List, Optional
 
 import archinfo
 from archinfo.arch_soot import ArchSoot
 
-from .address_translator import AT
-from .utils import ALIGN_UP, key_bisect_floor_key, key_bisect_insort_right
+from cle.address_translator import AT
+from cle.errors import CLECompatibilityError, CLEError, CLEFileNotFoundError, CLEOperationError
+from cle.memory import Clemory
+from cle.utils import ALIGN_UP, key_bisect_floor_key, key_bisect_insort_right, stream_or_path
 
-__all__ = ('Loader',)
+from .backends import ALL_BACKENDS, ELF, PE, Backend, Blob, ELFCore, MetaELF, Minidump
+from .backends.externs import ExternObject, KernelObject
+from .backends.tls import (
+    ELFCoreThreadManager,
+    ELFThreadManager,
+    MinidumpThreadManager,
+    PEThreadManager,
+    ThreadManager,
+    TLSObject,
+)
 
-l = logging.getLogger(name=__name__)
+__all__ = ("Loader",)
+
+log = logging.getLogger(name=__name__)
 
 
 class Loader:
@@ -68,18 +81,35 @@ class Loader:
 
     More keys are defined on a per-backend basis.
     """
-    # _main_binary_path: str
-    memory: Optional['Clemory']
-    main_object: Optional['Backend']
-    tls: Optional['ThreadManager']
 
-    def __init__(self, main_binary, auto_load_libs=True, concrete_target = None,
-                 force_load_libs=(), skip_libs=(),
-                 main_opts=None, lib_opts=None, ld_path=(), use_system_libs=True,
-                 ignore_import_version_numbers=True, case_insensitive=False, rebase_granularity=0x100000,
-                 except_missing_libs=False, aslr=False, perform_relocations=True, load_debug_info=False,
-                 page_size=0x1, preload_libs=(), arch=None):
-        if hasattr(main_binary, 'seek') and hasattr(main_binary, 'read'):
+    # _main_binary_path: str
+    memory: Optional["Clemory"]
+    main_object: Optional["Backend"]
+    tls: Optional["ThreadManager"]
+
+    def __init__(
+        self,
+        main_binary,
+        auto_load_libs=True,
+        concrete_target=None,
+        force_load_libs=(),
+        skip_libs=(),
+        main_opts=None,
+        lib_opts=None,
+        ld_path=(),
+        use_system_libs=True,
+        ignore_import_version_numbers=True,
+        case_insensitive=False,
+        rebase_granularity=0x100000,
+        except_missing_libs=False,
+        aslr=False,
+        perform_relocations=True,
+        load_debug_info=False,
+        page_size=0x1,
+        preload_libs=(),
+        arch=None,
+    ):
+        if hasattr(main_binary, "seek") and hasattr(main_binary, "read"):
             self._main_binary_path = None
             self._main_binary_stream = main_binary
         else:
@@ -110,11 +140,14 @@ class Loader:
         self._perform_relocations = perform_relocations
 
         # case insensitivity setup
-        if sys.platform == 'win32': # TODO: a real check for case insensitive filesystems
-            if self._main_binary_path: self._main_binary_path = self._main_binary_path.lower()
+        if sys.platform == "win32":  # TODO: a real check for case insensitive filesystems
+            if self._main_binary_path:
+                self._main_binary_path = self._main_binary_path.lower()
             force_load_libs = [x.lower() if type(x) is str else x for x in force_load_libs]
-            for x in list(self._satisfied_deps): self._satisfied_deps[x.lower()] = self._satisfied_deps[x]
-            for x in list(self._lib_opts): self._lib_opts[x.lower()] = self._lib_opts[x]
+            for x in list(self._satisfied_deps):
+                self._satisfied_deps[x.lower()] = self._satisfied_deps[x]
+            for x in list(self._lib_opts):
+                self._lib_opts[x.lower()] = self._lib_opts[x]
             self._custom_ld_path = [x.lower() for x in self._custom_ld_path]
 
         self.aslr = aslr
@@ -122,32 +155,37 @@ class Loader:
         self.memory = None
         self.main_object = None
         self.tls = None
-        self._kernel_object = None # type: Optional[KernelObject]
-        self._extern_object = None # type: Optional[ExternObject]
+        self._kernel_object: Optional[KernelObject] = None
+        self._extern_object: Optional[ExternObject] = None
         self.shared_objects = OrderedDict()
-        self.all_objects = []  # type: List[Backend]
+        self.all_objects: List[Backend] = []
         self.requested_names = set()
         if arch is not None:
-            self._main_opts.update({'arch': arch})
+            self._main_opts.update({"arch": arch})
         self.preload_libs = []
-        self.initial_load_objects = self._internal_load(main_binary, *preload_libs, *force_load_libs, preloading=(main_binary, *preload_libs))
+        self.initial_load_objects = self._internal_load(
+            main_binary, *preload_libs, *force_load_libs, preloading=(main_binary, *preload_libs)
+        )
 
         # cache
         self._last_object = None
 
         if self._extern_object and self._extern_object._warned_data_import:
-            l.warning('For more information about "Symbol was allocated without a known size", see https://docs.angr.io/extending-angr/environment#simdata')
+            log.warning(
+                'For more information about "Symbol was allocated without a known size",'
+                "see https://docs.angr.io/extending-angr/environment#simdata"
+            )
 
     # Basic functions and properties
 
     def close(self):
-        l.warning("You don't need to close the loader anymore :)")
+        log.warning("You don't need to close the loader anymore :)")
 
     def __repr__(self):
         if self._main_binary_stream is None:
-            return f'<Loaded {os.path.basename(self._main_binary_path)}, maps [{self.min_addr:#x}:{self.max_addr:#x}]>'
+            return f"<Loaded {os.path.basename(self._main_binary_path)}, maps [{self.min_addr:#x}:{self.max_addr:#x}]>"
         else:
-            return f'<Loaded from stream, maps [{self.min_addr:#x}:{self.max_addr:#x}]>'
+            return f"<Loaded from stream, maps [{self.min_addr:#x}:{self.max_addr:#x}]>"
 
     @property
     def max_addr(self):
@@ -213,9 +251,9 @@ class Loader:
             1) extern objects are a linked list. the one in loader._extern_object is the head of the list
             2) each round of explicit loads generates a new extern object if it has unresolved dependencies. this object
                has exactly the size necessary to hold all its exports.
-            3) All requests for size are passed down the chain until they reach an object which has the space to service it
-               or an object which has not yet been mapped. If all objects have been mapped and are full, a new extern object
-               is mapped with a fixed size.
+            3) All requests for size are passed down the chain until they reach an object which has the space to service
+               it or an object which has not yet been mapped. If all objects have been mapped and are full, a new extern
+               object is mapped with a fixed size.
         """
         if self._extern_object is None:
             if self.main_object.arch.bits < 32:
@@ -229,7 +267,7 @@ class Loader:
         return self._extern_object
 
     @property
-    def kernel_object(self) -> 'KernelObject':
+    def kernel_object(self) -> "KernelObject":
         """
         Return the object used to provide addresses to syscalls.
 
@@ -259,7 +297,7 @@ class Loader:
         """
         Return a set of every name that was requested as a shared object dependency but could not be loaded
         """
-        return self.requested_names - {k for k,v in self._satisfied_deps.items() if v is not False}
+        return self.requested_names - {k for k, v in self._satisfied_deps.items() if v is not False}
 
     @property
     def auto_load_libs(self):
@@ -272,7 +310,7 @@ class Loader:
         o = self.find_object_containing(addr)
 
         if o is None:
-            return 'not part of a loaded object'
+            return "not part of a loaded object"
 
         options = []
 
@@ -284,7 +322,7 @@ class Loader:
             if not sym.name or sym.is_import:
                 idx -= 1
                 continue
-            options.append((sym.relative_addr, '%s+' % sym.name))
+            options.append((sym.relative_addr, "%s+" % sym.name))
             break
 
         if isinstance(o, ELF):
@@ -293,21 +331,21 @@ class Loader:
             except ValueError:
                 pass
             else:
-                options.append((plt_addr, 'PLT.%s+' % plt_name))
+                options.append((plt_addr, "PLT.%s+" % plt_name))
 
-        options.append((0, 'offset '))
+        options.append((0, "offset "))
 
         if o.provides:
             objname = o.provides
         elif o.binary:
             objname = os.path.basename(o.binary)
         elif self.main_object is o:
-            objname = 'main binary'
+            objname = "main binary"
         else:
-            objname = 'object loaded from stream'
+            objname = "object loaded from stream"
 
         best_offset, best_prefix = max(options, key=lambda v: v[0])
-        return f'{best_prefix}{rva - best_offset:#x} in {objname} ({AT.from_va(addr, o).to_lva():#x})'
+        return f"{best_prefix}{rva - best_offset:#x} in {objname} ({AT.from_va(addr, o).to_lva():#x})"
 
     # Search functions
 
@@ -357,15 +395,17 @@ class Loader:
                 self._last_object = obj_
                 return obj_
             else:
-                raise CLEError('Unsupported memory type %s' % type(obj_.memory))
+                raise CLEError("Unsupported memory type %s" % type(obj_.memory))
 
         # check the cache first
-        if self._last_object is not None and \
-                self._last_object.min_addr <= addr <= self._last_object.max_addr:
-            if not membership_check: return self._last_object
-            if not self._last_object.has_memory: return self._last_object
+        if self._last_object is not None and self._last_object.min_addr <= addr <= self._last_object.max_addr:
+            if not membership_check:
+                return self._last_object
+            if not self._last_object.has_memory:
+                return self._last_object
             o = _check_object_memory(self._last_object)
-            if o: return o
+            if o:
+                return o
 
         if addr > self.max_addr or addr < self.min_addr:
             return None
@@ -534,7 +574,9 @@ class Loader:
                 n = next(i)
                 peeks.append((n, i))
         while peeks:
-            element = min(peeks, key=lambda x: x[0].rebased_addr) # if we don't do this it might crash on comparing iterators
+            element = min(
+                peeks, key=lambda x: x[0].rebased_addr
+            )  # if we don't do this it might crash on comparing iterators
             n, i = element
             idx = peeks.index(element)
             yield n
@@ -619,7 +661,7 @@ class Loader:
         try:
             return self._internal_load(spec)
         except CLEFileNotFoundError as e:
-            l.warning("Dynamic load failed: %r", e)
+            log.warning("Dynamic load failed: %r", e)
             return None
 
     def get_loader_symbolic_constraints(self):
@@ -635,16 +677,15 @@ class Loader:
             claripy = None
 
         if not claripy:
-            l.error("Please install claripy to get symbolic constraints")
+            log.error("Please install claripy to get symbolic constraints")
             return []
         outputlist = []
         for obj in self.all_objects:
-            #TODO Fix Symbolic for tls whatever
+            # TODO Fix Symbolic for tls whatever
             if obj.aslr and isinstance(obj.mapped_base_symbolic, claripy.ast.BV):
                 outputlist.append(obj.mapped_base_symbolic == obj.mapped_base)
         return outputlist
 
-
     # Private stuff
 
     @staticmethod
@@ -653,7 +694,7 @@ class Loader:
         ld can have different names such as ld-2.19.so or ld-linux-x86-64.so.2 depending on symlinks and whatnot.
         This determines if `name` is a suitable candidate for ld.
         """
-        return 'ld.so' in name or 'ld64.so' in name or 'ld-linux' in name
+        return "ld.so" in name or "ld64.so" in name or "ld-linux" in name
 
     def _internal_load(self, *args, preloading=()):
         """
@@ -681,12 +722,14 @@ class Loader:
         objects = []
         preload_objects = []
         dependencies = []
-        cached_failures = set() # this assumes that the load path is global and immutable by the time we enter this func
+        cached_failures = (
+            set()
+        )  # this assumes that the load path is global and immutable by the time we enter this func
 
         for main_spec in args:
             is_preloading = any(spec is main_spec for spec in preloading)
             if self.find_object(main_spec, extra_objects=objects) is not None:
-                l.info("Skipping load request %s - already loaded", main_spec)
+                log.info("Skipping load request %s - already loaded", main_spec)
                 continue
             obj = self._load_object_isolated(main_spec)
             objects.append(obj)
@@ -698,7 +741,11 @@ class Loader:
                 self.main_object = obj
                 self.memory = Clemory(obj.arch, root=True)
 
-                chk_obj = self.main_object if isinstance(self.main_object, ELFCore) or not self.main_object.child_objects else self.main_object.child_objects[0]
+                chk_obj = (
+                    self.main_object
+                    if isinstance(self.main_object, ELFCore) or not self.main_object.child_objects
+                    else self.main_object.child_objects[0]
+                )
                 if isinstance(chk_obj, ELFCore):
                     self.tls = ELFCoreThreadManager(self, obj.arch)
                 elif isinstance(obj, Minidump):
@@ -714,21 +761,20 @@ class Loader:
                 self.preload_libs.append(obj)
                 preload_objects.append(obj)
 
-
         while self._auto_load_libs and dependencies:
             spec = dependencies.pop(0)
             if spec in cached_failures:
-                l.debug("Skipping implicit dependency %s - cached failure", spec)
+                log.debug("Skipping implicit dependency %s - cached failure", spec)
                 continue
             if self.find_object(spec, extra_objects=objects) is not None:
-                l.debug("Skipping implicit dependency %s - already loaded", spec)
+                log.debug("Skipping implicit dependency %s - already loaded", spec)
                 continue
 
             try:
-                l.info("Loading %s...", spec)
+                log.info("Loading %s...", spec)
                 obj = self._load_object_isolated(spec)  # loading dependencies
             except CLEFileNotFoundError:
-                l.info("... not found")
+                log.info("... not found")
                 cached_failures.add(spec)
                 if self._except_missing_libs:
                     raise
@@ -738,7 +784,7 @@ class Loader:
             objects.extend(obj.child_objects)
             dependencies.extend(obj.deps)
 
-            if type(self.tls) is ThreadManager:   # ... java
+            if type(self.tls) is ThreadManager:  # ... java
                 if isinstance(obj, MetaELF):
                     self.tls = ELFThreadManager(self, obj.arch)
                 elif isinstance(obj, PE):
@@ -748,14 +794,21 @@ class Loader:
         # produce dependency-ordered list of objects and soname map
 
         ordered_objects = []
-        soname_mapping = OrderedDict((obj.provides if not self._ignore_import_version_numbers else obj.provides.rstrip('.0123456789'), obj) for obj in objects if obj.provides)
+        soname_mapping = OrderedDict(
+            (obj.provides if not self._ignore_import_version_numbers else obj.provides.rstrip(".0123456789"), obj)
+            for obj in objects
+            if obj.provides
+        )
         seen = set()
+
         def visit(obj):
             if id(obj) in seen:
                 return
             seen.add(id(obj))
 
-            stripped_deps = [dep if not self._ignore_import_version_numbers else dep.rstrip('.0123456789') for dep in obj.deps]
+            stripped_deps = [
+                dep if not self._ignore_import_version_numbers else dep.rstrip(".0123456789") for dep in obj.deps
+            ]
             dep_objs = [soname_mapping[dep_name] for dep_name in stripped_deps if dep_name in soname_mapping]
             for dep_obj in dep_objs:
                 visit(dep_obj)
@@ -779,13 +832,17 @@ class Loader:
         # link everything
         if self._perform_relocations:
             for obj in ordered_objects:
-                l.info("Linking %s", obj.binary)
+                log.info("Linking %s", obj.binary)
                 sibling_objs = list(obj.parent_object.child_objects) if obj.parent_object is not None else []
-                stripped_deps = [dep if not self._ignore_import_version_numbers else dep.rstrip('.0123456789') for dep in obj.deps]
+                stripped_deps = [
+                    dep if not self._ignore_import_version_numbers else dep.rstrip(".0123456789") for dep in obj.deps
+                ]
                 dep_objs = [soname_mapping[dep_name] for dep_name in stripped_deps if dep_name in soname_mapping]
                 main_objs = [self.main_object] if self.main_object is not obj else []
                 for reloc in obj.relocs:
-                    reloc.resolve_symbol(main_objs + preload_objects + sibling_objs + dep_objs + [obj], extern_object=extern_obj)
+                    reloc.resolve_symbol(
+                        main_objs + preload_objects + sibling_objs + dep_objs + [obj], extern_object=extern_obj
+                    )
 
         # if the extern object was used, add it to the list of objects we're mapping
         # also add it to the linked list of extern objects
@@ -835,14 +892,14 @@ class Loader:
         # STEP 1: identify file
         if isinstance(spec, Backend):
             return spec
-        elif hasattr(spec, 'read') and hasattr(spec, 'seek'):
+        elif hasattr(spec, "read") and hasattr(spec, "seek"):
             binary_stream = spec
             binary = None
             close = False
         elif type(spec) in (bytes, str):
-            binary = self._search_load_path(spec) # this is allowed to cheat and do partial static loading
-            l.debug("... using full path %s", binary)
-            binary_stream = open(binary, 'rb')
+            binary = self._search_load_path(spec)  # this is allowed to cheat and do partial static loading
+            log.debug("... using full path %s", binary)
+            binary_stream = open(binary, "rb")
             close = True
         else:
             raise CLEError("Bad library specification: %s" % spec)
@@ -852,7 +909,9 @@ class Loader:
             if self.main_object is None:
                 options = dict(self._main_opts)
             else:
-                for ident in self._possible_idents(binary_stream if binary is None else binary): # also allowed to cheat
+                for ident in self._possible_idents(
+                    binary_stream if binary is None else binary
+                ):  # also allowed to cheat
                     if ident in self._lib_opts:
                         options = dict(self._lib_opts[ident])
                         break
@@ -860,15 +919,17 @@ class Loader:
                     options = {}
 
             # STEP 3: identify backend
-            backend_spec = options.pop('backend', None)
+            backend_spec = options.pop("backend", None)
             backend_cls = self._backend_resolver(backend_spec)
             if backend_cls is None:
                 backend_cls = self._static_backend(binary_stream if binary is None else binary)
             if backend_cls is None:
-                raise CLECompatibilityError("Unable to find a loader backend for %s.  Perhaps try the 'blob' loader?" % spec)
+                raise CLECompatibilityError(
+                    "Unable to find a loader backend for %s.  Perhaps try the 'blob' loader?" % spec
+                )
 
             # STEP 4: LOAD!
-            l.debug("... loading with %s", backend_cls)
+            log.debug("... loading with %s", backend_cls)
 
             result = backend_cls(binary, binary_stream, is_main_bin=self.main_object is None, loader=self, **options)
             result.close()
@@ -877,7 +938,7 @@ class Loader:
             if close:
                 binary_stream.close()
 
-    def _map_object(self, obj: 'Backend'):
+    def _map_object(self, obj: "Backend"):
         """
         This will integrate the object into the global address space, but will not perform relocations.
         """
@@ -891,24 +952,32 @@ class Loader:
             elif not obj.is_main_bin:
                 base_addr = self._find_safe_rebase_addr(obj_size)
             else:
-                l.warning("The main binary is a position-independent executable. "
-                          "It is being loaded with a base address of 0x400000.")
+                log.warning(
+                    "The main binary is a position-independent executable. "
+                    "It is being loaded with a base address of 0x400000."
+                )
                 base_addr = 0x400000
 
             obj.rebase(base_addr)
         else:
-            if obj._custom_base_addr is not None and obj.linked_base != obj._custom_base_addr and not isinstance(obj, Blob):
-                l.warning("%s: base_addr was specified but the object is not PIC. "
-                          "specify force_rebase=True to override", obj.binary_basename)
+            if (
+                obj._custom_base_addr is not None
+                and obj.linked_base != obj._custom_base_addr
+                and not isinstance(obj, Blob)
+            ):
+                log.warning(
+                    "%s: base_addr was specified but the object is not PIC. " "specify force_rebase=True to override",
+                    obj.binary_basename,
+                )
             base_addr = obj.linked_base
             if not self._is_range_free(obj.linked_base, obj_size):
-                raise CLEError("Position-DEPENDENT object %s cannot be loaded at %#x"% (obj.binary, base_addr))
+                raise CLEError(f"Position-DEPENDENT object {obj.binary} cannot be loaded at {base_addr:#x}")
 
         assert obj.mapped_base >= 0
 
         if obj.has_memory:
             assert obj.min_addr <= obj.max_addr
-            l.info("Mapping %s at %#x", obj.binary, base_addr)
+            log.info("Mapping %s at %#x", obj.binary, base_addr)
             self.memory.add_backer(base_addr, obj.memory)
         obj._is_mapped = True
         key_bisect_insort_right(self.all_objects, obj, keyfunc=lambda o: o.min_addr)
@@ -921,7 +990,7 @@ class Loader:
         overlap with anything already loaded.
         """
         # this assumes that self.main_object exists, which should... definitely be safe
-        if self.main_object.arch.bits < 32 or self.main_object.max_addr >= 2**(self.main_object.arch.bits-1):
+        if self.main_object.arch.bits < 32 or self.main_object.max_addr >= 2 ** (self.main_object.arch.bits - 1):
             # HACK: On small arches, we should be more aggressive in packing stuff in.
             gap_start = 0
         else:
@@ -990,7 +1059,7 @@ class Loader:
         The only check performed is whether the file exists or not.
         """
         dirs = []
-        dirs.extend(self._custom_ld_path)                   # if we say dirs = blah, we modify the original
+        dirs.extend(self._custom_ld_path)  # if we say dirs = blah, we modify the original
 
         if self.main_object is not None:
             # add path of main binary
@@ -1002,40 +1071,43 @@ class Loader:
                 # ... extend with load path of native libraries
                 dirs.extend(self.main_object.extra_load_path)
                 if self._use_system_libs:
-                    l.debug("Path to system libraries (usually added as dependencies of JNI libs) needs "
-                            "to be specified manually, by using the custom_ld_path option.")
+                    log.debug(
+                        "Path to system libraries (usually added as dependencies of JNI libs) needs "
+                        "to be specified manually, by using the custom_ld_path option."
+                    )
             # add path of system libraries
             if self._use_system_libs and not is_arch_soot:
                 # Ideally this should be taken into account for each shared
                 # object, not just the main object.
                 dirs.extend(self.main_object.extra_load_path)
-                if sys.platform.startswith('linux'):
+                if sys.platform.startswith("linux"):
                     dirs.extend(self.main_object.arch.library_search_path())
-                elif sys.platform.startswith('openbsd'):
+                elif sys.platform.startswith("openbsd"):
                     dirs.extend(self.main_object.arch.library_search_path())
-                    dirs.extend(['/usr/local/lib', '/usr/X11R6/lib'])
-                elif sys.platform == 'win32':
-                    native_dirs = os.environ['PATH'].split(';')
+                    dirs.extend(["/usr/local/lib", "/usr/X11R6/lib"])
+                elif sys.platform == "win32":
+                    native_dirs = os.environ["PATH"].split(";")
 
                     # simulate the wow64 filesystem redirect, working around the fact that WE may be impacted by it as
                     # a 32-bit python process.......
-                    python_is_32bit = platform.architecture()[0] == '32bit'
+                    python_is_32bit = platform.architecture()[0] == "32bit"
                     guest_is_32bit = self.main_object.arch.bits == 32
 
                     if python_is_32bit != guest_is_32bit:
-                        redirect_dir = os.path.join(os.environ['SystemRoot'], 'system32').lower()
-                        target_dir = os.path.join(os.environ['SystemRoot'], 'SysWOW64' if guest_is_32bit else 'sysnative')
+                        redirect_dir = os.path.join(os.environ["SystemRoot"], "system32").lower()
+                        target_dir = os.path.join(
+                            os.environ["SystemRoot"], "SysWOW64" if guest_is_32bit else "sysnative"
+                        )
                         i = 0
                         while i < len(native_dirs):
                             if native_dirs[i].lower().startswith(redirect_dir):
                                 # replace the access to System32 with SysWOW64 or sysnative
-                                native_dirs[i] = target_dir + native_dirs[i][len(target_dir):]
+                                native_dirs[i] = target_dir + native_dirs[i][len(target_dir) :]
                             i += 1
 
                     dirs.extend(native_dirs)
 
-        dirs.append('.')
-
+        dirs.append(".")
 
         if self._case_insensitive:
             spec = spec.lower()
@@ -1054,9 +1126,10 @@ class Loader:
                 try:
                     for libname in os.listdir(libdir):
                         ilibname = libname.lower() if self._case_insensitive else libname
-                        if ilibname.strip('.0123456789') == spec.strip('.0123456789'):
+                        if ilibname.strip(".0123456789") == spec.strip(".0123456789"):
                             yield os.path.realpath(os.path.join(libdir, libname))
-                except OSError: pass
+                except OSError:
+                    pass
 
     @classmethod
     def _path_insensitive(cls, path):
@@ -1065,14 +1138,14 @@ class Loader:
 
         From https://stackoverflow.com/a/8462613
         """
-        if path == '' or os.path.exists(path):
+        if path == "" or os.path.exists(path):
             return path
         base = os.path.basename(path)  # may be a directory or a file
         dirname = os.path.dirname(path)
-        suffix = ''
+        suffix = ""
         if not base:  # dir ends with a slash?
             if len(dirname) < len(path):
-                suffix = path[:len(path) - len(dirname)]
+                suffix = path[: len(path) - len(dirname)]
             base = os.path.basename(dirname)
             dirname = os.path.dirname(dirname)
         if not os.path.exists(dirname):
@@ -1102,27 +1175,27 @@ class Loader:
             if spec.provides is not None:
                 yield spec.provides
                 if self._ignore_import_version_numbers:
-                    yield spec.provides.rstrip('.0123456789')
+                    yield spec.provides.rstrip(".0123456789")
             if spec.binary:
                 yield spec.binary
                 yield os.path.basename(spec.binary)
-                yield os.path.basename(spec.binary).split('.')[0]
+                yield os.path.basename(spec.binary).split(".")[0]
                 if self._ignore_import_version_numbers:
-                    yield os.path.basename(spec.binary).rstrip('.0123456789')
-        elif hasattr(spec, 'read') and hasattr(spec, 'seek'):
+                    yield os.path.basename(spec.binary).rstrip(".0123456789")
+        elif hasattr(spec, "read") and hasattr(spec, "seek"):
             backend_cls = self._static_backend(spec, ignore_hints=True)
             if backend_cls is not None:
                 soname = backend_cls.extract_soname(spec)
                 if soname is not None:
                     yield soname
                     if self._ignore_import_version_numbers:
-                        yield soname.rstrip('.0123456789')
+                        yield soname.rstrip(".0123456789")
         elif type(spec) in (bytes, str):
             yield spec
             yield os.path.basename(spec)
-            yield os.path.basename(spec).split('.')[0]
+            yield os.path.basename(spec).split(".")[0]
             if self._ignore_import_version_numbers:
-                yield os.path.basename(spec).rstrip('.0123456789')
+                yield os.path.basename(spec).rstrip(".0123456789")
 
             if os.path.exists(spec):
                 backend_cls = self._static_backend(spec, ignore_hints=True)
@@ -1131,9 +1204,9 @@ class Loader:
                     if soname is not None:
                         yield soname
                         if self._ignore_import_version_numbers:
-                            yield soname.rstrip('.0123456789')
+                            yield soname.rstrip(".0123456789")
 
-        if not lowercase and (sys.platform == 'win32' or self._case_insensitive):
+        if not lowercase and (sys.platform == "win32" or self._case_insensitive):
             for name in self._possible_idents(spec, lowercase=True):
                 yield name.lower()
 
@@ -1147,7 +1220,7 @@ class Loader:
         if not ignore_hints:
             for ident in self._possible_idents(spec):
                 try:
-                    return self._backend_resolver(self._lib_opts[ident]['backend'])
+                    return self._backend_resolver(self._lib_opts[ident]["backend"])
                 except KeyError:
                     pass
 
@@ -1167,12 +1240,22 @@ class Loader:
         elif backend is None:
             return default
         else:
-            raise CLEError('Invalid backend: %s' % backend)
+            raise CLEError("Invalid backend: %s" % backend)
 
+    #
+    # Memory data loading methods
+    #
 
-from .errors import CLEError, CLEFileNotFoundError, CLECompatibilityError, CLEOperationError
-from .memory import Clemory
-from .backends import MetaELF, ELF, PE, ELFCore, Minidump, Blob, ALL_BACKENDS, Backend
-from .backends.tls import ThreadManager, ELFThreadManager, PEThreadManager, ELFCoreThreadManager, MinidumpThreadManager, TLSObject
-from .backends.externs import ExternObject, KernelObject
-from .utils import stream_or_path
+    def fast_memory_load_pointer(self, addr: int, size: Optional[int] = None) -> Optional[int]:
+        """
+        Perform a fast memory loading of a pointer.
+
+        :param addr:    Address to read from.
+        :param size:    Size of the pointer. Default to machine-word size.
+        :return:        A pointer or None if the address does not exist.
+        """
+
+        try:
+            return self.memory.unpack_word(addr, size=size)
+        except KeyError:
+            return None
